#define MOZ_SOURCE_STAMP b22f7ff530bb0cedd0b38d5a837ab3482071e086
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/b22f7ff530bb0cedd0b38d5a837ab3482071e086
