/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=8 sts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef NSCTOOLTIPTEXTPROVIDER_H
#define NSCTOOLTIPTEXTPROVIDER_H

#include "nsITooltipTextProvider.h"

#define NS_TOOLTIPTEXTPROVIDER_CONTRACTID \
  "@mozilla.org/embedcomp/tooltiptextprovider;1"
#define NS_DEFAULTTOOLTIPTEXTPROVIDER_CONTRACTID \
  "@mozilla.org/embedcomp/default-tooltiptextprovider;1"

#endif
